@@ -22,7 +22,6 @@
 #include "ConnectionPool.h"
 #include "LogUtils.h"
 #include "NamespaceName.h"
-#include "ServiceNameResolver.h"
 #include "TopicName.h"
 
 DECLARE_LOG_OBJECT()
@@ -87,9 +86,9 @@ auto BinaryProtoLookupService::findBroker(const std::string& address, bool autho
                                                 << ", from " << cnx->cnxString());
                 if (data->shouldProxyThroughServiceUrl()) {
                     // logicalAddress is the proxy's address, we should still connect through proxy
-                    promise->setValue({responseBrokerAddress, address});
+                    promise->setValue({responseBrokerAddress, address, true});
                 } else {
-                    promise->setValue({responseBrokerAddress, responseBrokerAddress});
+                    promise->setValue({responseBrokerAddress, responseBrokerAddress, false});
                 }
             }
         });
