@@ -29,6 +29,7 @@
 #include "Future.h"
 #include "LookupDataResult.h"
 #include "ProtoApiEnums.h"
+#include "ServiceNameResolver.h"
 
 namespace pulsar {
 using NamespaceTopicsPtr = std::shared_ptr<std::vector<std::string>>;
@@ -42,6 +43,7 @@ class LookupService {
     struct LookupResult {
         std::string logicalAddress;
         std::string physicalAddress;
+        bool proxyThroughServiceUrl;
 
         friend std::ostream& operator<<(std::ostream& os, const LookupResult& lookupResult) {
             return os << "logical address: " << lookupResult.logicalAddress
@@ -85,6 +87,8 @@ class LookupService {
     virtual Future<Result, SchemaInfo> getSchema(const TopicNamePtr& topicName,
                                                  const std::string& version = "") = 0;
 
+    virtual ServiceNameResolver& getServiceNameResolver() = 0;
+
     virtual ~LookupService() {}
 
     virtual void close() {}
