@@ -58,7 +58,7 @@ PartitionedProducerImpl::PartitionedProducerImpl(ClientImplPtr client, const Top
     if (partitionsUpdateInterval > 0) {
         listenerExecutor_ = client->getListenerExecutorProvider()->get();
         partitionsUpdateTimer_ = listenerExecutor_->createDeadlineTimer();
-        partitionsUpdateInterval_ = boost::posix_time::seconds(partitionsUpdateInterval);
+        partitionsUpdateInterval_ = std::chrono::seconds(partitionsUpdateInterval);
         lookupServicePtr_ = client->getLookup();
     }
 }
@@ -69,7 +69,7 @@ MessageRoutingPolicyPtr PartitionedProducerImpl::getMessageRouter() {
             return std::make_shared<RoundRobinMessageRouter>(
                 conf_.getHashingScheme(), conf_.getBatchingEnabled(), conf_.getBatchingMaxMessages(),
                 conf_.getBatchingMaxAllowedSizeInBytes(),
-                boost::posix_time::milliseconds(conf_.getBatchingMaxPublishDelayMs()));
+                std::chrono::milliseconds(conf_.getBatchingMaxPublishDelayMs()));
         case ProducerConfiguration::CustomPartition:
             return conf_.getMessageRouterPtr();
         case ProducerConfiguration::UseSinglePartition:
@@ -92,10 +92,12 @@ unsigned int PartitionedProducerImpl::getNumPartitionsWithLock() const {
     return getNumPartitions();
 }
 
-ProducerImplPtr PartitionedProducerImpl::newInternalProducer(unsigned int partition, bool lazy) {
+ProducerImplPtr PartitionedProducerImpl::newInternalProducer(unsigned int partition, bool lazy,
+                                                             bool retryOnCreationError) {
     using namespace std::placeholders;
     auto client = client_.lock();
-    auto producer = std::make_shared<ProducerImpl>(client, *topicName_, conf_, interceptors_, partition);
+    auto producer = std::make_shared<ProducerImpl>(client, *topicName_, conf_, interceptors_, partition,
+                                                   retryOnCreationError);
     if (!client) {
         return producer;
     }
@@ -127,13 +129,13 @@ void PartitionedProducerImpl::start() {
 
         for (unsigned int i = 0; i < getNumPartitions(); i++) {
             bool lazy = (short)i != partition;
-            producers_.push_back(newInternalProducer(i, lazy));
+            producers_.push_back(newInternalProducer(i, lazy, false));
         }
 
         producers_[partition]->start();
     } else {
         for (unsigned int i = 0; i < getNumPartitions(); i++) {
-            producers_.push_back(newInternalProducer(i, false));
+            producers_.push_back(newInternalProducer(i, false, false));
         }
 
         for (ProducerList::const_iterator prod = producers_.begin(); prod != producers_.end(); prod++) {
@@ -420,7 +422,7 @@ void PartitionedProducerImpl::flushAsync(FlushCallback callback) {
 void PartitionedProducerImpl::runPartitionUpdateTask() {
     auto weakSelf = weak_from_this();
     partitionsUpdateTimer_->expires_from_now(partitionsUpdateInterval_);
-    partitionsUpdateTimer_->async_wait([weakSelf](const boost::system::error_code& ec) {
+    partitionsUpdateTimer_->async_wait([weakSelf](const ASIO_ERROR& ec) {
         auto self = weakSelf.lock();
         if (self) {
             self->getPartitionMetadata();
@@ -461,7 +463,7 @@ void PartitionedProducerImpl::handleGetPartitions(Result result,
             for (unsigned int i = currentNumPartitions; i < newNumPartitions; i++) {
                 ProducerImplPtr producer;
                 try {
-                    producer = newInternalProducer(i, lazy);
+                    producer = newInternalProducer(i, lazy, true);
                 } catch (const std::runtime_error& e) {
                     LOG_ERROR("Failed to create producer for partition " << i << ": " << e.what());
                     producers.clear();
@@ -522,7 +524,7 @@ uint64_t PartitionedProducerImpl::getNumberOfConnectedProducer() {
 
 void PartitionedProducerImpl::cancelTimers() noexcept {
     if (partitionsUpdateTimer_) {
-        boost::system::error_code ec;
+        ASIO_ERROR ec;
         partitionsUpdateTimer_->cancel(ec);
     }
 }
