@@ -20,7 +20,7 @@
 
 #include <pulsar/MessageIdBuilder.h>
 
-#include <boost/date_time/posix_time/posix_time.hpp>
+#include <chrono>
 
 #include "BatchMessageContainer.h"
 #include "BatchMessageKeyBasedContainer.h"
@@ -46,9 +46,11 @@
 namespace pulsar {
 DECLARE_LOG_OBJECT()
 
+using std::chrono::milliseconds;
+
 ProducerImpl::ProducerImpl(ClientImplPtr client, const TopicName& topicName,
                            const ProducerConfiguration& conf, const ProducerInterceptorsPtr& interceptors,
-                           int32_t partition)
+                           int32_t partition, bool retryOnCreationError)
     : HandlerBase(client, (partition < 0) ? topicName.toString() : topicName.getTopicPartitionName(partition),
                   Backoff(milliseconds(client->getClientConfig().getInitialBackoffIntervalMs()),
                           milliseconds(client->getClientConfig().getMaxBackoffIntervalMs()),
@@ -67,7 +69,8 @@ ProducerImpl::ProducerImpl(ClientImplPtr client, const TopicName& topicName,
       dataKeyRefreshTask_(*executor_, 4 * 60 * 60 * 1000),
       memoryLimitController_(client->getMemoryLimitController()),
       chunkingEnabled_(conf_.isChunkingEnabled() && topicName.isPersistent() && !conf_.getBatchingEnabled()),
-      interceptors_(interceptors) {
+      interceptors_(interceptors),
+      retryOnCreationError_(retryOnCreationError) {
     LOG_DEBUG("ProducerName - " << producerName_ << " Created producer on topic " << topic()
                                 << " id: " << producerId_);
     if (!producerName_.empty()) {
@@ -141,8 +144,12 @@ Future<Result, bool> ProducerImpl::connectionOpened(const ClientConnectionPtr& c
         return promise.getFuture();
     }
 
+    LOG_INFO("Creating producer for topic:" << topic() << ", producerName:" << producerName_ << " on "
+                                            << cnx->cnxString());
     ClientImplPtr client = client_.lock();
-    int requestId = client->newRequestId();
+    cnx->registerProducer(producerId_, shared_from_this());
+
+    long requestId = client->newRequestId();
 
     SharedBuffer cmd = Commands::newProducer(topic(), producerId_, producerName_, requestId,
                                              conf_.getProperties(), conf_.getSchema(), epoch_,
@@ -152,6 +159,7 @@ Future<Result, bool> ProducerImpl::connectionOpened(const ClientConnectionPtr& c
 
     // Keep a reference to ensure object is kept alive.
     auto self = shared_from_this();
+    setFirstRequestIdAfterConnect(requestId);
     cnx->sendRequestWithId(cmd, requestId)
         .addListener([this, self, cnx, promise](Result result, const ResponseData& responseData) {
             Result handleResult = handleCreateProducer(cnx, result, responseData);
@@ -173,7 +181,7 @@ void ProducerImpl::connectionFailed(Result result) {
         // if producers are lazy, then they should always try to restart
         // so don't change the state and allow reconnections
         return;
-    } else if (producerCreatedPromise_.setFailed(result)) {
+    } else if (!isResultRetryable(result) && producerCreatedPromise_.setFailed(result)) {
         state_ = Failed;
     }
 }
@@ -211,7 +219,6 @@ Result ProducerImpl::handleCreateProducer(const ClientConnectionPtr& cnx, Result
         // set the cnx pointer so that new messages will be sent immediately
         LOG_INFO(getName() << "Created producer on broker " << cnx->cnxString());
 
-        cnx->registerProducer(producerId_, shared_from_this());
         producerName_ = responseData.producerName;
         schemaVersion_ = responseData.schemaVersion;
         producerStr_ = "[" + topic() + ", " + producerName_ + "] ";
@@ -273,7 +280,7 @@ Result ProducerImpl::handleCreateProducer(const ClientConnectionPtr& cnx, Result
             lock.unlock();
             producerCreatedPromise_.setFailed(result);
             handleResult = result;
-        } else if (producerCreatedPromise_.isComplete()) {
+        } else if (producerCreatedPromise_.isComplete() || retryOnCreationError_) {
             if (result == ResultProducerBlockedQuotaExceededException) {
                 LOG_WARN(getName() << "Backlog is exceeded on topic. Sending exception to producer");
                 failPendingMessages(ResultProducerBlockedQuotaExceededException, false);
@@ -464,7 +471,7 @@ void ProducerImpl::sendAsync(const Message& msg, SendCallback callback) {
     Producer producer = Producer(shared_from_this());
     auto interceptorMessage = interceptors_->beforeSend(producer, msg);
 
-    const auto now = boost::posix_time::microsec_clock::universal_time();
+    const auto now = TimeUtils::now();
     auto self = shared_from_this();
     sendAsyncWithStatsUpdate(interceptorMessage, [this, self, now, callback, producer, interceptorMessage](
                                                      Result result, const MessageId& messageId) {
@@ -563,10 +570,9 @@ void ProducerImpl::sendAsyncWithStatsUpdate(const Message& msg, SendCallback&& c
         bool isFirstMessage = batchMessageContainer_->isFirstMessageToAdd(msg);
         bool isFull = batchMessageContainer_->add(msg, callback);
         if (isFirstMessage) {
-            batchTimer_->expires_from_now(
-                boost::posix_time::milliseconds(conf_.getBatchingMaxPublishDelayMs()));
+            batchTimer_->expires_from_now(milliseconds(conf_.getBatchingMaxPublishDelayMs()));
             auto weakSelf = weak_from_this();
-            batchTimer_->async_wait([this, weakSelf](const boost::system::error_code& ec) {
+            batchTimer_->async_wait([this, weakSelf](const ASIO_ERROR& ec) {
                 auto self = weakSelf.lock();
                 if (!self) {
                     return;
@@ -628,7 +634,6 @@ void ProducerImpl::sendAsyncWithStatsUpdate(const Message& msg, SendCallback&& c
                 const uint32_t msgHeadersAndPayloadSize = msgMetadataSize + payloadSize;
                 if (msgHeadersAndPayloadSize > maxMessageSize) {
                     lock.unlock();
-                    releaseSemaphoreForSendOp(*op);
                     LOG_WARN(getName()
                              << " - compressed Message size " << msgHeadersAndPayloadSize << " cannot exceed "
                              << maxMessageSize << " bytes unless chunking is enabled");
@@ -823,14 +828,14 @@ Future<Result, ProducerImplBaseWeakPtr> ProducerImpl::getProducerCreatedFuture()
 
 uint64_t ProducerImpl::getProducerId() const { return producerId_; }
 
-void ProducerImpl::handleSendTimeout(const boost::system::error_code& err) {
+void ProducerImpl::handleSendTimeout(const ASIO_ERROR& err) {
     const auto state = state_.load();
     if (state != Pending && state != Ready) {
         return;
     }
     Lock lock(mutex_);
 
-    if (err == boost::asio::error::operation_aborted) {
+    if (err == ASIO::error::operation_aborted) {
         LOG_DEBUG(getName() << "Timer cancelled: " << err.message());
         return;
     } else if (err) {
@@ -846,8 +851,8 @@ void ProducerImpl::handleSendTimeout(const boost::system::error_code& err) {
     } else {
         // If there is at least one message, calculate the diff between the message timeout and
         // the current time.
-        time_duration diff = pendingMessagesQueue_.front()->timeout - TimeUtils::now();
-        if (diff.total_milliseconds() <= 0) {
+        auto diff = pendingMessagesQueue_.front()->timeout - TimeUtils::now();
+        if (toMillis(diff) <= 0) {
             // The diff is less than or equal to zero, meaning that the message has been expired.
             LOG_DEBUG(getName() << "Timer expired. Calling timeout callbacks.");
             pendingMessages = getPendingCallbacksWhenFailed();
@@ -855,7 +860,7 @@ void ProducerImpl::handleSendTimeout(const boost::system::error_code& err) {
             asyncWaitSendTimeout(milliseconds(conf_.getSendTimeout()));
         } else {
             // The diff is greater than zero, set the timeout to the diff value
-            LOG_DEBUG(getName() << "Timer hasn't expired yet, setting new timeout " << diff);
+            LOG_DEBUG(getName() << "Timer hasn't expired yet, setting new timeout " << diff.count());
             asyncWaitSendTimeout(diff);
         }
     }
@@ -969,12 +974,15 @@ bool ProducerImpl::encryptMessage(proto::MessageMetadata& metadata, SharedBuffer
                                encryptedPayload);
 }
 
-void ProducerImpl::disconnectProducer() {
-    LOG_INFO("Broker notification of Closed producer: " << producerId_);
+void ProducerImpl::disconnectProducer(const boost::optional<std::string>& assignedBrokerUrl) {
+    LOG_INFO("Broker notification of Closed producer: "
+             << producerId_ << (assignedBrokerUrl ? (" assignedBrokerUrl: " + assignedBrokerUrl.get()) : ""));
     resetCnx();
-    scheduleReconnection();
+    scheduleReconnection(assignedBrokerUrl);
 }
 
+void ProducerImpl::disconnectProducer() { disconnectProducer(boost::none); }
+
 void ProducerImpl::start() {
     HandlerBase::start();
 
@@ -999,7 +1007,7 @@ void ProducerImpl::shutdown() {
 
 void ProducerImpl::cancelTimers() noexcept {
     dataKeyRefreshTask_.stop();
-    boost::system::error_code ec;
+    ASIO_ERROR ec;
     batchTimer_->cancel(ec);
     sendTimer_->cancel(ec);
 }
@@ -1025,7 +1033,7 @@ void ProducerImpl::asyncWaitSendTimeout(DurationType expiryTime) {
     sendTimer_->expires_from_now(expiryTime);
 
     auto weakSelf = weak_from_this();
-    sendTimer_->async_wait([weakSelf](const boost::system::error_code& err) {
+    sendTimer_->async_wait([weakSelf](const ASIO_ERROR& err) {
         auto self = weakSelf.lock();
         if (self) {
             std::static_pointer_cast<ProducerImpl>(self)->handleSendTimeout(err);
