@@ -22,11 +22,14 @@
 
 #include <algorithm>
 #include <boost/exception/all.hpp>
+#include <chrono>
 #include <future>
+#include <memory>
 #include <stdexcept>
 
 #include "HttpHelper.h"
 #include "PulsarFriend.h"
+#include "PulsarWrapper.h"
 #include "lib/BinaryProtoLookupService.h"
 #include "lib/ClientConnection.h"
 #include "lib/ConnectionPool.h"
@@ -48,7 +51,10 @@ namespace pulsar {
 
 class LookupServiceTest : public ::testing::TestWithParam<std::string> {
    public:
-    void SetUp() override { client_ = Client{GetParam()}; }
+    void SetUp() override {
+        serviceUrl_ = GetParam();
+        client_ = Client{serviceUrl_};
+    }
     void TearDown() override { client_.close(); }
 
     template <typename T>
@@ -63,6 +69,7 @@ class LookupServiceTest : public ::testing::TestWithParam<std::string> {
     }
 
    protected:
+    std::string serviceUrl_;
     Client client_{httpLookupUrl};
 };
 
@@ -77,8 +84,7 @@ TEST(LookupServiceTest, basicLookup) {
     ClientConfiguration conf;
     ExecutorServiceProviderPtr ioExecutorProvider_(std::make_shared<ExecutorServiceProvider>(1));
     ConnectionPool pool_(conf, ioExecutorProvider_, authData, "");
-    ServiceNameResolver serviceNameResolver(url);
-    BinaryProtoLookupService lookupService(serviceNameResolver, pool_, conf);
+    BinaryProtoLookupService lookupService(url, pool_, conf);
 
     TopicNamePtr topicName = TopicName::get("topic");
 
@@ -141,26 +147,24 @@ static void testMultiAddresses(LookupService& lookupService) {
 
 TEST(LookupServiceTest, testMultiAddresses) {
     ConnectionPool pool({}, std::make_shared<ExecutorServiceProvider>(1), AuthFactory::Disabled(), "");
-    ServiceNameResolver serviceNameResolver("pulsar://localhost,localhost:9999");
     ClientConfiguration conf;
-    BinaryProtoLookupService binaryLookupService(serviceNameResolver, pool, conf);
+    BinaryProtoLookupService binaryLookupService("pulsar://localhost,localhost:9999", pool, conf);
     testMultiAddresses(binaryLookupService);
 
     // HTTPLookupService calls shared_from_this() internally, we must create a shared pointer to test
-    ServiceNameResolver serviceNameResolverForHttp("http://localhost,localhost:9999");
     auto httpLookupServicePtr = std::make_shared<HTTPLookupService>(
-        std::ref(serviceNameResolverForHttp), ClientConfiguration{}, AuthFactory::Disabled());
+        "http://localhost,localhost:9999", ClientConfiguration{}, AuthFactory::Disabled());
     testMultiAddresses(*httpLookupServicePtr);
 }
 TEST(LookupServiceTest, testRetry) {
     auto executorProvider = std::make_shared<ExecutorServiceProvider>(1);
     ConnectionPool pool({}, executorProvider, AuthFactory::Disabled(), "");
-    ServiceNameResolver serviceNameResolver("pulsar://localhost:9999,localhost");
     ClientConfiguration conf;
 
     auto lookupService = RetryableLookupService::create(
-        std::make_shared<BinaryProtoLookupService>(serviceNameResolver, pool, conf), 30 /* seconds */,
-        executorProvider);
+        std::make_shared<BinaryProtoLookupService>("pulsar://localhost:9999,localhost", pool, conf),
+        std::chrono::seconds(30), executorProvider);
+    ServiceNameResolver& serviceNameResolver = lookupService->getServiceNameResolver();
 
     PulsarFriend::setServiceUrlIndex(serviceNameResolver, 0);
     auto topicNamePtr = TopicName::get("lookup-service-test-retry");
@@ -189,13 +193,13 @@ TEST(LookupServiceTest, testRetry) {
 TEST(LookupServiceTest, testTimeout) {
     auto executorProvider = std::make_shared<ExecutorServiceProvider>(1);
     ConnectionPool pool({}, executorProvider, AuthFactory::Disabled(), "");
-    ServiceNameResolver serviceNameResolver("pulsar://localhost:9990,localhost:9902,localhost:9904");
     ClientConfiguration conf;
 
     constexpr int timeoutInSeconds = 2;
     auto lookupService = RetryableLookupService::create(
-        std::make_shared<BinaryProtoLookupService>(serviceNameResolver, pool, conf), timeoutInSeconds,
-        executorProvider);
+        std::make_shared<BinaryProtoLookupService>("pulsar://localhost:9990,localhost:9902,localhost:9904",
+                                                   pool, conf),
+        std::chrono::seconds(timeoutInSeconds), executorProvider);
     auto topicNamePtr = TopicName::get("lookup-service-test-retry");
 
     decltype(std::chrono::high_resolution_clock::now()) startTime;
@@ -431,13 +435,38 @@ TEST_P(LookupServiceTest, testGetSchemaByVersion) {
     producer2.close();
 }
 
+TEST_P(LookupServiceTest, testGetSchemaTimeout) {
+    if (serviceUrl_.find("pulsar://") == std::string::npos) {
+        // HTTP request timeout can only be configured with seconds
+        return;
+    }
+    const auto topic = "lookup-service-test-get-schema-timeout";
+    Producer producer;
+    ProducerConfiguration producerConf;
+    producerConf.setSchema(SchemaInfo(STRING, "", ""));
+    ASSERT_EQ(ResultOk, client_.createProducer(topic, producerConf, producer));
+    ASSERT_EQ(ResultOk, producer.send(MessageBuilder().setContent("msg").build()));
+    client_.close();
+
+    ClientConfiguration conf;
+    PulsarWrapper::deref(conf).operationTimeout = std::chrono::nanoseconds(1);
+    client_ = Client{serviceUrl_, conf};
+    auto promise = std::make_shared<std::promise<Result>>();
+    client_.getSchemaInfoAsync(topic, 0L,
+                               [promise](Result result, const SchemaInfo&) { promise->set_value(result); });
+    auto future = promise->get_future();
+    ASSERT_EQ(future.wait_for(std::chrono::milliseconds(100)), std::future_status::ready);
+    ASSERT_EQ(future.get(), ResultTimeout);
+    client_.close();
+}
+
 INSTANTIATE_TEST_SUITE_P(Pulsar, LookupServiceTest, ::testing::Values(binaryLookupUrl, httpLookupUrl));
 
 class BinaryProtoLookupServiceRedirectTestHelper : public BinaryProtoLookupService {
    public:
-    BinaryProtoLookupServiceRedirectTestHelper(ServiceNameResolver& serviceNameResolver, ConnectionPool& pool,
+    BinaryProtoLookupServiceRedirectTestHelper(const std::string& serviceUrl, ConnectionPool& pool,
                                                const ClientConfiguration& clientConfiguration)
-        : BinaryProtoLookupService(serviceNameResolver, pool, clientConfiguration) {}
+        : BinaryProtoLookupService(serviceUrl, pool, clientConfiguration) {}
 
     LookupResultFuture findBroker(const std::string& address, bool authoritative, const std::string& topic,
                                   size_t redirectCount) {
@@ -452,14 +481,13 @@ TEST(LookupServiceTest, testRedirectionLimit) {
     conf.setMaxLookupRedirects(redirect_limit);
     ExecutorServiceProviderPtr ioExecutorProvider_(std::make_shared<ExecutorServiceProvider>(1));
     ConnectionPool pool_(conf, ioExecutorProvider_, authData, "");
-    std::string url = "pulsar://localhost:6650";
-    ServiceNameResolver serviceNameResolver(url);
-    BinaryProtoLookupServiceRedirectTestHelper lookupService(serviceNameResolver, pool_, conf);
+    string url = "pulsar://localhost:6650";
+    BinaryProtoLookupServiceRedirectTestHelper lookupService(url, pool_, conf);
 
     const auto topicNamePtr = TopicName::get("topic");
     for (auto idx = 0; idx < redirect_limit + 5; ++idx) {
-        auto future =
-            lookupService.findBroker(serviceNameResolver.resolveHost(), false, topicNamePtr->toString(), idx);
+        auto future = lookupService.findBroker(lookupService.getServiceNameResolver().resolveHost(), false,
+                                               topicNamePtr->toString(), idx);
         LookupService::LookupResult lookupResult;
         auto result = future.get(lookupResult);
 
